@@ -17,7 +17,7 @@
 #endif
   def (name)
   def (title)
-#ifdef BG_IMAGE_FROM_FILE
+#if BG_IMAGE_FROM_FILE
   def (path)
   def (backgroundPixmap)
 #endif
@@ -54,20 +54,21 @@
 #ifdef PRINTPIPE
   def (print_pipe)
 #endif
-#ifdef USE_XIM
+#if USE_XIM
   def (preeditType)
   def (inputMethod)
 #endif
-#ifdef ENABLE_TRANSPARENCY
+#if ENABLE_TRANSPARENCY
   def (transparent)
   def (shade)
   def (blurradius)
 #endif
 #if XFT
-  def (depth)
   def (buffered)
 #endif
 #if ENABLE_FRILLS
+  def (depth)
+  def (visual)
   def (transient_for)
   def (override_redirect)
   def (pty_fd)
@@ -80,7 +81,7 @@
   def (cursorUnderline)
   def (urgentOnBell)
 #endif
-#ifdef BUILTIN_GLYPHS
+#if BUILTIN_GLYPHS
   def (skipBuiltinGlyphs)
 #endif
 #if CURSOR_BLINK
@@ -103,7 +104,7 @@
   def (secondaryScreen)
   def (secondaryScroll)
 #endif
-#ifdef OFF_FOCUS_FADING
+#if OFF_FOCUS_FADING
   def (fade)
 #endif
 #if ENABLE_PERL
@@ -116,9 +117,6 @@
   def (iso14755)
   def (iso14755_52)
 #endif
-#ifdef HAVE_AFTERIMAGE
-  def (blendtype)
-#endif
-#ifdef ENABLE_EWMH
+#if ENABLE_EWMH
   def (iconfile)
 #endif
